// file: signal_detector/test/grow_0.cc
//

// isip include files
//
#include "../signal_detector.h"
#include "../signal_detector_constants.h"

// function: grow_arrays_cc
//
// arguments:
//  float_4** arrays_p: (input/output) arrays to be resized
//  int_4 num_arrays_p: (input) number of arrays
//  int_4& cur_size_p: (input/output) number of elements in each array
//			       (arrays are assumed to be the same length)
//  int_4 grow_size_p: (input) amount to increment the array
//
// return:
//  an int_4 containing the new array size
//
// this method grows arrays by creating a new copy.
//
int_4 grow_arrays_cc(float_4** arrays_p, int_4 num_arrays_p,
		     int_4& cur_size_p, int_4 grow_size_p) {

  // allocate new, larger, buffers
  //
  float_4** new_arrays = new float_4*[num_arrays_p];

  for (int_4 counter=0; counter<num_arrays_p; counter++)
    {new_arrays[counter] = new float_4[cur_size_p + grow_size_p];}

  // transfer the contents from the existing buffer
  //
  for (int_4 na=(int_4)0; na<num_arrays_p; na++) {
    for (int_4 counter=0; counter<cur_size_p; counter++) {
      (new_arrays[na])[counter] = (arrays_p[na])[counter];
    }
  }
  cur_size_p += grow_size_p;

  // swap pointers
  //
  for (int_4 na=(int_4)0; na<num_arrays_p; na++) {
    delete [] arrays_p[na];
    arrays_p[na] = new_arrays[na];
  }
  delete [] new_arrays;

  // exit gracefully
  //
  return cur_size_p;
}

