// file: signal_detector/v2.1/src/signal_detector.h
//
// this file defines the signal_detector class
//

// make sure definitions are only made once
//
#ifndef __ISIP_SIGNAL_DETECTOR
#define __ISIP_SIGNAL_DETECTOR

// system include files
//
#include <stdio.h>

// isip include files
//
#include "integral.h"

#include "src_base/signal_detector_base.h"

// Signal_detector: a class that is used to find the start/stop times
//	of speech in speech data streams
//
class Signal_detector  {
  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:

  // the specific algorithm
  //
  Signal_detector_base* sigd_d;
  char_1* implementation_name_d;

  // initialization flags
  //
  logical_1 init_d;

  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:

  // constructors/destructors
  //
  Signal_detector();
  ~Signal_detector();

  // required methods
  //
  char_1* name_cc();
  logical_1 debug_cc(FILE* fp, char_1* message);
 
  //print parameters  
  //
  logical_1 print_parameters_cc(char_1* output_file);
   
  // load parameters
  //
  logical_1 load_parameters_cc(char_1* parameter_file);
 
  // initialization methods
  //
  logical_1 init_cc();
  logical_1 flush_cc();
  logical_1 reset_cc();
  
  // computational methods
  //
  logical_1 process_cc(void* signal);
  
  // status methods
  //
  logical_1 get_endpoints_cc(float_4& t1, float_4& t2);

  // miscellaneous functions
  //  
  int_4 get_nbytes_to_read_cc();

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:

  // object creation-related methods
  //
  logical_1 create_object_from_name_cc(char_1* implementation_name);
  logical_1 get_implementation_name_cc(char_1* param_file);
  volatile void error_handler_cc(char* method_name,char* message);

  // miscellaneous string processing methods
  //
  int_4 Signal_detector::string_length_cc(char_1* string);

};

// end of include file
#endif














