// file: signal_detector/src_00/sigd00_init_0.cc
//

// isip include files
//
#include "../signal_detector.h"
#include "signal_detector_00.h"
#include "signal_detector_00_constants.h"

// method: init_cc
//
// arguments: none
//
// return: logical_1
//
logical_1 Signal_detector_00::init_cc() {
   
  // convert time to samples and frames
  //
  i_frame_dur_d = (int_4)rint(frame_dur_d * sample_freq_d);
  i_window_dur_d = (int_4)rint(window_dur_d * sample_freq_d);
  i_min_utt_dur_d = (int_4)rint(min_utt_dur_d / frame_dur_d);
  i_min_utt_sep_d = (int_4)rint(min_utt_sep_d / frame_dur_d);

  // compute (1/N)
  //
  win_dur_scale_factor_d = 1.0 / (float_4)i_window_dur_d;
  
  // allocate memory
  //
  if (x_pre_d != (float_4*)NULL)
    {delete [] x_pre_d;}
  x_pre_d = new float_4[i_window_dur_d];
  
  // clear filter memory
  //
  nf_d = (int_4)0;
  xn1_d = (float_4)0.0;

  // initialize adaptation constants
  //
  noise_thresh_d = nom_noise_lev_d;
  sig_thresh_d = nom_sig_lev_d;
  
  // create a state machine
  //
  if (states_d != (int_4*)NULL)
    {delete [] states_d;}
  states_d = new int_4[SIGD_HISTORY];
  
  if (durations_d != (int_4*)NULL)
    {delete [] durations_d;}
  durations_d = new int_4[SIGD_NSTATES];
  
  for (int_4 ns=(int_4)0; ns<SIGD_HISTORY; ns++)
    {states_d[ns] = SIGD_STATE_NOIS;}
  for (int_4 ns=(int_4)0; ns<SIGD_NSTATES; ns++)
    {durations_d[ns] = (int_4)0;}
  
  // create a buffer to use to delay decisions about the start
  // of an utterance. note that we initialize it to the noise state.
  //
  cb_len_d = (int_4)rint(max_utt_dur_d / frame_dur_d);
  rlse_frm_d = (int_4)0;
  cur_frm_d = rlse_frm_d + i_min_utt_sep_d;
  
  if (cb_d != (int_4*)NULL)
    {delete [] cb_d;}
  cb_d = new int_4[cb_len_d];
  
  for (int_4 nf=rlse_frm_d; nf<=cur_frm_d; nf++) {
    cb_d[nf] = SIGD_STATE_NOIS;
  }
  
  // establish patterns to be used in the pattern matching software
  //
  sig_patn_len_d = i_min_utt_dur_d;
  nse_patn_len_d = i_min_utt_sep_d;
  
  if (sig_patn_d != (int_4*)NULL)
    {delete [] sig_patn_d;}
  sig_patn_d = new int_4[sig_patn_len_d];
  if (nse_patn_d != (int_4*)NULL)
    {delete [] nse_patn_d;}
  nse_patn_d = new int_4[nse_patn_len_d];
  
  for (int_4 nf=0; nf<sig_patn_len_d; nf++) {
    sig_patn_d[nf] = SIGD_STATE_SIGN;
  }
  for (int_4 nf=0; nf<nse_patn_len_d; nf++) {
    nse_patn_d[nf] = SIGD_STATE_NOIS;
  }
  
  // clear utterance detection parameters
  //
  utt_in_progress_d = ISIP_FALSE;
  utt_beg_cb_indx_d = (int_4)-1;
  utt_end_cb_indx_d = (int_4)-1;
  utt_beg_d = (int_4)-1;
  utt_end_d = (int_4)-1;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}


