// file: signal_detector/src_00/sigd00_proc_0.cc
//
//
// isip include files
//

#include "signal_detector_00.h"
#include "signal_detector_00_constants.h"

// method: process_cc
//
// arguments: void* signal buffer
//
// return: logical_1
//
logical_1 Signal_detector_00::process_cc(void* buf_p) {
 
  // for now, only deal with a sample size of 2 bytes
  //
  if (sample_size_d != (int_4)2)
    {error_handler_cc((char_1*)"process_cc", (char_1*)"invalid sample size");}

  // compute the rms value
  //
  float_4 egy = compute_energy_cc(buf_p);
  
  // check this value against the thresholds by sending it to the
  // state machine
  //
  int_4 cur_state = sm_advance_cc(egy);
  cb_add_cc(cur_state);

  if (debug_level_d == ISIP_DEBUG_BRIEF) {
    fprintf(stdout, "%f %f %f %f %ld\n",
	    nf_d*frame_dur_d, egy, sig_thresh_d, noise_thresh_d,
	    cb_d[cur_frm_d]);
  }

  // adjust circular buffer, smoothing out short bursts of frames
  // that fall below the signal threshold during an utterance
  // that don't meet the utterance duration constraints
  //
  cb_smooth_signal_cc();

  // adjust circular buffer, smoothing out short bursts of frames
  // that rise above the noise level that don't meet the utterance
  // constraints
  //
  cb_smooth_noise_cc();

  // smooth out short bursts of signal that don't meet the
  // minimum signal duration constraint
  //
  cb_remove_short_signal_bursts_cc();

  // smooth out short bursts of noise to free buffer space
  //
  cb_remove_short_noise_bursts_cc();

  // remove extremely long runs of SIGD_STATE_NOIS -
  //  note that this simply frees memory in the circular buffer. if
  //  you are processing extremely long runs of data with large gaps of
  //  silence, the circular buffer can overflow. hence, we remove these
  //  gaps by freeing space at the end of the buffer.
  //
  cb_remove_long_noise_bursts_cc();

  // smooth out bursts that occur during the transition between NOIS and SIGN
  //
  cb_smooth_transitions_cc();

  if (debug_level_d == ISIP_DEBUG_FULL) {
    cb_dump_cc(stdout, (char_1*)"buffer after smoothing");
  }

  if (debug_level_d > ISIP_DEBUG_BRIEF) {
    if (nf_d > i_min_utt_sep_d) {
      int_4 indx = cb_increment_cc(cur_frm_d, -i_min_utt_sep_d);
      fprintf(stdout, "%f %f %f %f %ld\n",
	      (nf_d - i_min_utt_sep_d)*frame_dur_d, egy,
	      sig_thresh_d, noise_thresh_d, cb_d[indx]);
    }
  }

  // do we have an utterance in progress?
  //  note that this call is made merely to make sure the flag is set
  //
  if (utt_in_progress_d == ISIP_FALSE)
    {utt_in_progress_d = cb_utt_in_progress_cc();}

  if (utt_in_progress_d == ISIP_TRUE) {

    if (debug_level_d > ISIP_DEBUG_BRIEF) {
      fprintf(stdout, "start of utterance detected at frame %ld\n", nf_d);
    }

    // check for an utterance end
    //
    if (cb_utt_not_in_progress_cc() == ISIP_TRUE) {

      if (debug_level_d > ISIP_DEBUG_BRIEF) {
	fprintf(stdout, "end of utterance detected at frame %ld\n", nf_d);
      }

      // clear utt in progress flag
      //
      utt_in_progress_d = ISIP_FALSE;

      // generate rough utterance endpoints
      //
      get_rough_endpoints_cc();

      if (debug_level_d == ISIP_DEBUG_FULL) {
	fprintf(stdout, "rough endpoints from %ld to %ld\n",
		utt_beg_cb_indx_d, utt_end_cb_indx_d);
      }

      // refine these endpoints
      //
      get_refined_endpoints_cc();

      if (debug_level_d == ISIP_DEBUG_FULL) {
	fprintf(stdout, "refined endpoints from %ld to %ld\n",
		utt_beg_d, utt_end_d);
      }

      // update rlse_frm_d
      //
      cb_release_cc(cur_frm_d);

      if (debug_level_d == ISIP_DEBUG_FULL) {
	fprintf(stdout, "new release frame index is %ld\n", rlse_frm_d);
      }
    }
  }

  // update adaptative thresholds
  //
  if (update_thresholds_cc(egy) == ISIP_FALSE) {
    error_handler_cc((char_1*)"process_cc",
		     (char_1*)"error updating thresholds");
  }

  // purge the circular buffer - another way to free space
  //
  cb_purge_cc();

  // update counters
  //
  nf_d++;

  // exit gracefully
  //
  return ISIP_TRUE;
}
