// file: /signal_detector/src_00/sigdp00_cb_10.cc
//

// isip include files
//
#include "signal_detector_00.h"
#include "signal_detector_00_constants.h"

// method: cb_remove_long_noise_bursts_cc
//
// arguments: none
//
// return:
//  a logical_1 indicating the status
//
// this method removes long bursts of noise. note that:
// 
//  - it works forward in time starting at rlse_frm_d
//  - it can only remove i_min_utt_sep_d frames of noise when it finds
//    twice this much
// 
logical_1 Signal_detector_00::cb_remove_long_noise_bursts_cc() {

  // define the required number of consecutive states of NOIS
  //
  int_4 num_states_req = 2 * i_min_utt_sep_d;

  // start at rlse_frm_d, and count the number of states assigned to NOIS
  //
  int_4 nf = rlse_frm_d;

  for (int_4 counter=0; counter<num_states_req; counter++) {

    // check the value of the state
    //
    if (cb_d[nf] != SIGD_STATE_NOIS)
      {return ISIP_FALSE;}

    // update counters
    //
    nf = cb_increment_cc(nf, (int_4)1);
    if (nf == cur_frm_d)
      {return ISIP_FALSE;}
  }

  // looks like we can free some buffer space - this is done
  //  by simply resetting rlse_frm_d. note that we only clear
  //  1/2 the space found, because we need the other half for
  //  utterance detection.
  //
  rlse_frm_d = cb_increment_cc(rlse_frm_d, i_min_utt_sep_d);

  // exit gracefully - we have successfully cleared some space
  //
  return ISIP_TRUE;
}
