// file: /signal_detector/src_00/sigdp00_cb_12.cc
//

// isip include files
//
#include "signal_detector_00.h"
#include "signal_detector_00_constants.h"

// method: cb_purge_cc
//
// arguments: none
//
// return:
//  a logical_1 indicating the status
//
// this method releases space in the circular buffer for reuse.
// sometimes, isolated runs of SIGN less than min_utt_dur_d get retained in
// the circular buffer. if these aren't removed, the buffer will overflow.
// assuming the circular buffer is long, by the time they hit the back-end of
// the buffer, they will never be declared part of an utterance.
//
logical_1 Signal_detector_00::cb_purge_cc() {

  // start at the back-end of the buffer
  //
  if (cb_diff_cc(cur_frm_d, rlse_frm_d) < 4*i_min_utt_sep_d)
    {return ISIP_FALSE;}

  // scan for a noise pattern
  //
  int_4 index = rlse_frm_d;
  int_4 last_index = cb_increment_cc(rlse_frm_d, 2*i_min_utt_sep_d);

  // find each occurrence of a non-NOIS state
  //
  while (index != last_index) {

    // find a non-NOIS state
    //
    if (cb_d[index] != SIGD_STATE_NOIS) {

      // count the number of consecutive non-NOIS states
      //
      int_4 pos = index;
      int_4 count = (int_4)0;
	
      while (cb_d[pos] != SIGD_STATE_NOIS) {
	count++;
	if (count == i_min_utt_dur_d)
	  {break;}
	pos = cb_increment_cc(pos, (int_4)1);
      }
      
      // if min_utt_dur_d is not satisfied, kill them
      //
      if (count < i_min_utt_dur_d) {
	pos = index;
	while (cb_d[pos] != SIGD_STATE_NOIS) {
	  cb_d[pos] = SIGD_STATE_NOIS;
	  pos = cb_increment_cc(pos, (int_4)1);
	}
      }
    }

    // increment counters
    //
    index = cb_increment_cc(index, (int_4)1);
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
