// file:/signal_detector/src_00/sigdp00_cb_14.cc
//

// isip include files
//
#include "signal_detector_00.h"
#include "signal_detector_00_constants.h"

// method: cb_smooth_transitions_cc
//
// arguments: none
//
// return:
//  a logical_1 indicating the status
//
// this method turns short bursts of energy transitions that occur prior to
// min_utt_dur of SIGN states into either NOIS or SIGN.
//
// this algorithm works as follows:
//
//  (1) checks from the current position if there are min_utt_dur consecutive
//      frames assigned to state SIGN.  the time at which these SIGN state
//      first occur is called 't2'.
//  (2) scans backward to find another min_utt_dur consecutive frames
//      assigned to state NOIS previous to the frames in step (1).  the end of
//      these NOIS states is called 't1'.
//  (3) for all frames between t1 and t2, a sum is accumulated, with a SIGN
//      state adding 1, NOIS state adding 0, and transition states adding 1/2.
//  (4) if this sum is greater that the total number of NOIS states in this
//      interval, then all state in this interval are changed to SIGN, else
//      all states are changed to NOIS.
//
logical_1 Signal_detector_00::cb_smooth_transitions_cc() {

  // check from the current position for min_utt_dur of SIGN
  //
  int_4 counter = 0;
  int_4 nf = cur_frm_d;

  while (counter < i_min_utt_dur_d) {
    if (cb_d[nf] != SIGD_STATE_SIGN)
      {return ISIP_FALSE;}
    counter++;
    nf = cb_increment_cc(nf,(int_4)-1);
  }
  int_4 t2 = nf;
  
  // scan backwards for another pattern of SIGD_STATE_NOIS
  // that is min_utt_sep_d long
  //
  int_4 t1 = cb_scan_for_pattern_cc(nse_patn_d, i_min_utt_sep_d, nf);
  if (t1 < (int_4)0)
    {return ISIP_FALSE;}

  // accumulate as stated in (3)
  //
  float_4 sum_sign = (float_4)0.0;
  float_4 sum_nois = (float_4)0.0;

  for (counter=t1; counter<t2; counter++) {
    if (cb_d[counter] == SIGD_STATE_SIGN)
      {sum_sign += SIGD_SIGN_WEIGHT;}
    else if (cb_d[counter] == SIGD_STATE_NOIS)
      {sum_nois += SIGD_NOIS_WEIGHT;}
    else
      {sum_sign += SIGD_TRAN_WEIGHT;}
  }

  // make a decision on whether it is signal or noise
  //
  int_4 new_val;

  if (sum_sign > sum_nois)
    {new_val = SIGD_STATE_SIGN;}
  else
    {new_val = SIGD_STATE_NOIS;}

  // set the frames to the new value
  //
  t1 = cb_increment_cc(t1, (int_4)1);
  t2 = cb_increment_cc(t2, (int_4)1);

  for (counter=t1; counter<t2; counter++)
    {cb_d[counter] = new_val;}
  
  // exit gracefully
  //
  return ISIP_TRUE;
}




