// file: signal_detector/src_00/sigdp00_cb_5.cc
//

// isip include files
//
#include "signal_detector_00.h"
#include "signal_detector_00_constants.h"

// method: cb_utt_in_progress_cc
//
// arguments: none
//
// return:
//  a logical_1 indicating whether we have a start of utterance
//
// a start of utterance is defined as the point at which we have a
// run of SIGN states. this does not mean this is the final start time
// of the utterance (which is found from some postprocessing). it simply
// means we are confident there is an utterance in progress.
//
logical_1 Signal_detector_00::cb_utt_in_progress_cc() {

  // make sure the buffer is full
  //
  if (cb_diff_cc(cur_frm_d, rlse_frm_d) < i_min_utt_dur_d)
    {return ISIP_FALSE;}

  // seek backwards from cur_frm to rlse_frm looking for a run of states
  // equal to SIGN
  //
  int_4 nf = cur_frm_d;

  for (int_4 counter=0; counter<i_min_utt_dur_d; counter++) {

    // check the current frame
    //
    if (cb_d[nf] != SIGD_STATE_SIGN)
      {return ISIP_FALSE;}

    // decrement the counter
    //
    nf = cb_increment_cc(nf, (int_4)-1);
  }

  // exit gracefully - the signal is in the SIGN state
  //
  return ISIP_TRUE;
}
