// file: signal_detector/src_01/sigd01_cstr_0.cc
//

// isip include files
//
#include "signal_detector_01.h"
#include "signal_detector_01_constants.h"

// method: constructor
//
// arguments: none
//
// return: none
//
// this is the default constructor for the signal_detector_01 class
//
Signal_detector_01::Signal_detector_01() {

  // data format parameters
  //
  num_chan_d = SIGD_DEFAULT_NUMBER_OF_CHANNELS;
  sample_size_d = SIGD_DEFAULT_SAMPLE_SIZE;
  chan_d = SIGD_DEFAULT_CHANNEL_TO_BE_PROCESSED;

  // signal processing parameters
  //
  sample_freq_d = SIGD_DEFAULT_SAMPLE_FREQUENCY;
  frame_dur_d = SIGD_DEFAULT_FRAME_DURATION;
  window_dur_d = SIGD_DEFAULT_WINDOW_DURATION;
  preemphasis_d = SIGD_DEFAULT_PREEMPHASIS;

  // signal level-related energy parameters
  //
  nom_sig_lev_d = SIGD_DEFAULT_NOMINAL_SIGNAL_LEVEL;
  sig_delta_d = SIGD_DEFAULT_SIGNAL_ADAPTATION_DELTA;
  sig_adapt_d = SIGD_DEFAULT_SIGNAL_ADAPTATION_CONSTANT;

  // noise level-related energy parameters
  //
  nom_noise_lev_d = SIGD_DEFAULT_NOMINAL_NOISE_LEVEL;
  noise_delta_d = SIGD_DEFAULT_NOISE_ADAPTATION_DELTA;
  noise_adapt_d = SIGD_DEFAULT_NOISE_ADAPTATION_CONSTANT;
  noise_floor_d = SIGD_DEFAULT_NOISE_FLOOR;

  // utterance parameters
  //
  utt_delta_d = SIGD_DEFAULT_UTTERANCE_DELTA;
  min_utt_dur_d = SIGD_DEFAULT_MINIMUM_UTTERANCE_DURATION;
  min_utt_sep_d = SIGD_DEFAULT_MINIMUM_UTTERANCE_SEPARATION;
  max_utt_dur_d = SIGD_DEFAULT_MAXIMUM_UTTERANCE_DURATION;

  // debug level
  //
  debug_level_d = SIGD_DEFAULT_DEBUG_LEVEL;

  // initialize counters and such
  //
  xn1_d = (float_4)0.0;
  nf_d = (int_4)0;
  x_pre_d = (float_4*)NULL;

  // initialize adaptation constants
  //
  noise_thresh_d = noise_floor_d;
  sig_thresh_d = noise_floor_d;
  
  // create the state machine
  //
  states_d = (int_4*)NULL;
  durations_d = (int_4*)NULL;

  // create a circular buffer
  //
  cb_len_d = (int_4)0;
  cur_frm_d = (int_4)0;
  rlse_frm_d = (int_4)0;
  cb_d = (int_4*)NULL;
  zc_cb_d = (int_4*)NULL;

  // create the pattern matching buffers
  //
  sig_patn_len_d = (int_4)0;
  sig_patn_d = (int_4*)NULL;
  nse_patn_len_d = (int_4)0;
  nse_patn_d = (int_4*)NULL;

  // clear utterance detection parameters
  //
  utt_in_progress_d = ISIP_FALSE;
  utt_beg_cb_indx_d = (int_4)-1;
  utt_end_cb_indx_d = (int_4)-1;
  utt_beg_d = (int_4)-1;
  utt_end_d = (int_4)-1;
  refine_utt_beg_d = (int_4)-1;
  refine_utt_end_d = (int_4)-1;
  // exit gracefully
  //
}
