 // file:signal_detector/src_01/sigdp01_zc_rate_0.cc
//

// isip include files
//
#include "signal_detector_01.h"
#include "signal_detector_01_constants.h"

// method: compute_zc_rate_cc
//
// arguments:
//  void* buf_p: (input) signal buffer
//
// return:
//  a int_4 containing the zerocrossing rate value
//
// note that this method debiases the data
//
int_4 Signal_detector_01::compute_zc_rate_cc(void* buf_p) {

  // declare local variables
  //
  int_2* buf = (int_2*)buf_p;
  float_4* x_pre_off;
  float_4 xn;
  // find the offset in x_pre_d
  //
  if (i_window_dur_d > i_frame_dur_d)
    {x_pre_off = x_pre_d + (i_window_dur_d - i_frame_dur_d);}
  else
    {x_pre_off = x_pre_d;}

  // load the signal buffer
  //
  buf += chan_d;

  for (int_4 i=0; i<i_frame_dur_d; i++) {
    
    // grab a sample
    //
    xn = (float_4)(*buf) * SIGD_SAMPLE_SCALE_FACTOR;

    // preemphasize
    //
    *x_pre_off = xn - preemphasis_d * xn1_d;
    
    // shift the filter
    //
    xn1_d = xn;
    
    // increment counters
    //
    buf += num_chan_d;
    x_pre_off++;
  }

  // compute zc rate - do this backward so we don't have to worry
  //   when window duration is less than frame duration
  //
  int_4 num_crossings = (int_4)0;
  x_pre_off = x_pre_d + i_window_dur_d - 1;
  for (int_4 i=0; i<i_window_dur_d; i++){ 
   if ((*x_pre_off > (pzc_thresh*SIGD_SAMPLE_SCALE_FACTOR)) &&
	  (*(x_pre_off - 1)  < (nzc_thresh*SIGD_SAMPLE_SCALE_FACTOR))) 
	num_crossings++; 
   if ((*x_pre_off < (nzc_thresh*SIGD_SAMPLE_SCALE_FACTOR)) &&
          (*(x_pre_off - 1) > (pzc_thresh*SIGD_SAMPLE_SCALE_FACTOR)))
	num_crossings++;
    x_pre_off--;
    }

  // shift the signal
  //
  if (i_window_dur_d > i_frame_dur_d) {
    int_4 num_to_move = i_window_dur_d - i_frame_dur_d;
    x_pre_off = x_pre_d + i_frame_dur_d;
    memcpy(x_pre_d, x_pre_off, num_to_move*sizeof(float_4));
  }

  // exit gracefully - return the zero crossing rate
  //
  return (num_crossings);
}
