// file: signal_detector/src_01/signal_detector_01.h
//
// this file defines the signal_detector class
//

// make sure definitions are only made once
//
#ifndef __ISIP_SIGNAL_DETECTOR_01
#define __ISIP_SIGNAL_DETECTOR_01

// system include files
//
#include <stdio.h>

// isip include files
//
#include "../integral.h"

#include "../src_base/signal_detector_base.h"

// forward class definitions
//

// Signal_detector_01: a class that is used to find the start/stop times
//	of signals in data streams
//
class Signal_detector_01:public Signal_detector_base {

  //---------------------------------------------------------------------------
  //
  // protected data
  //

  //---------------------------------------------------------------------------
protected:
 
  float_4 algorithm;
  int_4 num_chan_d;
  int_4 sample_size_d;
  int_4 chan_d;
 
  // signal processing information
  //
  float_4 sample_freq_d;
  float_4 frame_dur_d;
  float_4 window_dur_d;
  float_4 preemphasis_d;
 
 // signal level-related energy parameters
 //
  float_4 nom_sig_lev_d;
  float_4 sig_delta_d;
  float_4 sig_adapt_d;
 
 // noise level-related energy parameters
 //
  float_4 nom_noise_lev_d;
  float_4 noise_delta_d;
  float_4 noise_adapt_d;
  float_4 noise_floor_d;

 // utterance-related parameters
  //
  float_4 utt_delta_d;
  float_4 min_utt_dur_d;
  float_4 min_utt_sep_d;
  float_4 max_utt_dur_d;


 // debugging parameters
  //
  int_4 debug_level_d;
 
 // integer versions of above parameters
 //
  int_4 i_frame_dur_d;
  int_4 i_window_dur_d;
  int_4 i_min_utt_dur_d;
  int_4 i_min_utt_sep_d;
 // filter-related parameters
 //
  float_4 xn1_d;

 // useful counters
 //
 int_4 nf_d;

 // signal buffers
 //
 float_4* x_pre_d;

 // useful scale factors
 //
 float_4 win_dur_scale_factor_d;

 // adaptive thresholds
  //
  float_4 noise_thresh_d;
  float_4 sig_thresh_d;

  // state machine
  //
  int_4* states_d;
  int_4* durations_d;

  // utterance detection buffer (implemented as a circular buffer)
  //
  int_4 cb_len_d;
  int_4 cur_frm_d;
  int_4 rlse_frm_d;
  int_4* cb_d;
  int_4* zc_cb_d;
  
  // utterance pattern matching parameters
  //
  int_4* sig_patn_d;
  int_4 sig_patn_len_d;

  int_4* nse_patn_d;
  int_4 nse_patn_len_d;

  // utterance state parameters
  //
  logical_1 utt_in_progress_d;
  int_4 utt_beg_cb_indx_d;
  int_4 utt_end_cb_indx_d;
  int_4 utt_beg_d;
  int_4 utt_end_d;
  int_4 refine_utt_beg_d;
  int_4 refine_utt_end_d;
  int_4 zc_utt_thresh;
  int_4 pzc_thresh;
  int_4 nzc_thresh;

  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------

public:

  // constructors/destructors
  //
  Signal_detector_01();
  virtual ~Signal_detector_01();
  
  // required methods
  //
  virtual char_1* name_cc();
  virtual int_4 get_nbytes_to_read_cc();
  virtual logical_1 debug_cc(FILE* fp, char_1* message);
  virtual logical_1 load_parameters_cc(char_1* parameter_file_p);
  virtual logical_1 print_parameters_cc(char_1* output_file_p);
  volatile void error_handler_cc(char_1* method_name,char_1* message);
  
  // initialization methods
  //
  virtual logical_1 init_cc();
  virtual logical_1 flush_cc();
  virtual logical_1 reset_cc();
  
  // computational methods
  //
  virtual logical_1 process_cc(void* signal);
  
  // status methods
  //
  virtual logical_1 get_endpoints_cc(float_4& t1, float_4& t2);
  

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------

  int_4 Signal_detector_01::string_length_cc(char_1* string);
  logical_1 Signal_detector_01::interpret_parameter_cc(char_1* name,
						    float_4& val);
 
  // energy and zerocrossing computation
  //
  float_4 compute_energy_cc(void* signal);
  int_4 compute_zc_rate_cc(void* signal);

  // threshold computation
  //
  logical_1 update_thresholds_cc(float_4 egy);
  
  // state machine related methods
  //
  logical_1 sm_reset_cc();
  int_4 sm_advance_cc(float_4 egy);

  // circular buffer related methods
  //
  logical_1 cb_dump_cc(FILE* fp, char_1* message);
  int_4 cb_increment_cc(int_4 index, int_4 value);
  int_4 cb_diff_cc(int_4 val1, int_4 val2);
  int_4 cb_add_cc(int_4 state);
  int_4 cb_scan_for_pattern_cc(int_4* pattern, int_4 len, int_4 start_frame);

  logical_1 cb_smooth_signal_cc();
  logical_1 cb_smooth_noise_cc();
  logical_1 cb_smooth_transitions_cc();
  logical_1 cb_utt_in_progress_cc();
  logical_1 cb_utt_not_in_progress_cc();

  logical_1 cb_remove_short_signal_bursts_cc();
  logical_1 cb_remove_short_noise_bursts_cc();
  logical_1 cb_remove_long_noise_bursts_cc();
  logical_1 cb_purge_cc();
  logical_1 cb_release_cc(int_4 frame_index);
   
  // zero crossing circular buffer stuff
  //
  int_4 zc_cb_increment_cc(int_4 index, int_4 value);
  int_4 zc_cb_diff_cc(int_4 val1, int_4 val2);
  int_4 zc_cb_add_cc(int_4 state);
  logical_1 zc_cb_release_cc(int_4);
  
  
  // endpoint processing
  //
  logical_1 get_rough_endpoints_cc();
  logical_1 get_refined_endpoints_cc();
  logical_1 get_more_refined_edpts_cc();

  // end of class
};

//
// end of include file
#endif






