// file: signal_detector/src_01/signal_detector_01_constants.h
//
// this file defines constants for the signal_detector_01 class
//

// make sure definitions are only made once
//
#ifndef __ISIP_SIGNAL_DETECTOR_01_CONSTANTS
#define __ISIP_SIGNAL_DETECTOR_01_CONSTANTS

// isip include files
//
#include "../integral_constants.h"

// define the class name
//
#define SIGD01_CLASS_NAME "energy_and_zcrate"

// declare default data format parameters
//
#define SIGD_DEFAULT_NUMBER_OF_CHANNELS 2
#define SIGD_DEFAULT_SAMPLE_SIZE 2
#define SIGD_DEFAULT_CHANNEL_TO_BE_PROCESSED 1

// signal processing parameters
//
#define SIGD_DEFAULT_SAMPLE_FREQUENCY 16000.0
#define SIGD_DEFAULT_FRAME_DURATION 0.020
#define SIGD_DEFAULT_WINDOW_DURATION 0.030
#define SIGD_DEFAULT_PREEMPHASIS 0.95

// signal level-related energy parameters
//
#define SIGD_DEFAULT_NOMINAL_SIGNAL_LEVEL -35.0
#define SIGD_DEFAULT_SIGNAL_ADAPTATION_DELTA 20.0
#define SIGD_DEFAULT_SIGNAL_ADAPTATION_CONSTANT 0.50


// noise level-related energy parameters
//
#define SIGD_DEFAULT_NOMINAL_NOISE_LEVEL -60.0
#define SIGD_DEFAULT_NOISE_ADAPTATION_DELTA 20.0
#define SIGD_DEFAULT_NOISE_ADAPTATION_CONSTANT 0.95
#define SIGD_DEFAULT_NOISE_FLOOR -65.0

// utterance parameters
//
#define SIGD_DEFAULT_UTTERANCE_DELTA 6.0
#define SIGD_DEFAULT_MINIMUM_UTTERANCE_DURATION 0.050
#define SIGD_DEFAULT_MINIMUM_UTTERANCE_SEPARATION 0.030
#define SIGD_DEFAULT_MAXIMUM_UTTERANCE_DURATION 10.0

// debug parameters
//
#define SIGD_DEFAULT_DEBUG_LEVEL ISIP_DEBUG_NONE

// string processing related parameters
//
#define SIGD_MAX_STRING_LENGTH 1024
#define SIGD_COMMENT_CHAR ISIP_POUND

// sample value scaling
//
#define SIGD_SAMPLE_SCALE_FACTOR 3.051850e-05

// state machine related stuff:
//  use a history of 3 states; create a state machine with 4 nodes
//
#define SIGD_HISTORY    (int_4)3
#define SIGD_NSTATES    (int_4)4
#define SIGD_STATE_NOIS (int_4)0
#define SIGD_STATE_NTOS (int_4)1
#define SIGD_STATE_STON (int_4)2
#define SIGD_STATE_SIGN (int_4)3

#define SIGD_SIGN_WEIGHT (float_4)1.0
#define SIGD_TRAN_WEIGHT (float_4)0.5
#define SIGD_NOIS_WEIGHT (float_4)0.0

#define SIGD_MAX_UTT_DUR 10.0

//
// end of file
#endif







