// file: signal_detector/src_01/sigdp01_cb_11.cc
//

// isip include files
//
#include "signal_detector_01.h"
#include "signal_detector_01_constants.h"

// method: cb_remove_short_noise_bursts_cc
//
// arguments: none
//
// return:
//  a logical_1 indicating the status
//
// this method removes short bursts of noise that do not meet the
// duration constraints. note that this method starts from the
// back of the circular buffer (rlse_frm_d).
//
// this algorithm works as follows:
//  (1) finds a run of states that toggle between NOIS and NTOS
//  (2) assigns them to NOIS
//
logical_1 Signal_detector_01::cb_remove_short_noise_bursts_cc() {

  // stay away from the beginning of the buffer
  //
  int_4 indx = cb_increment_cc(cur_frm_d, -i_min_utt_sep_d);

  // make sure we don't go past rlse_frm
  //
  if (cb_diff_cc(indx, rlse_frm_d) < (int_4)0)
    {return ISIP_FALSE;}

  // assign all NTOS states to NOIS (some of these slip through
  // all the other logic and prevent buffer space from being released)
  //
  while (indx != rlse_frm_d) {
    if (cb_d[indx] == SIGD_STATE_NTOS)
      {cb_d[indx] = SIGD_STATE_NOIS;}
    indx = cb_increment_cc(indx, (int_4)-1);
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
