// file: /signal_detector/src_01/sigdp01_cb_3.cc
//

// isip include files
//
#include "signal_detector_01.h"
#include "signal_detector_01_constants.h"

// method: cb_scan_for_pattern_cc
//
// arguments:
//  int_4* pattern_p: (input) pattern to be matched
//  int_4 len_p: (input) length of the pattern
//  int_4 start_frame_p: (input) position from which to start scan 
//
// return:
//  an int_4 indicating the position of the match
//  a value less than zero indicates no match
//
// this method scans the circular buffer for the first occurrence of the
// pattern supplied as an argument.
//
int_4 Signal_detector_01::cb_scan_for_pattern_cc(int_4* pattern_p, int_4 len_p,
					      int_4 start_frame_p) {

  // seek backwards from start_frame_p
  //
  int_4 nf_strt = start_frame_p;

  // find the stop index
  //
  int_4 nf_stop = rlse_frm_d;
  for(int_4 nf=0; nf<len_p; nf++) {
    nf_stop = cb_increment_cc(nf_stop, (int_4)1);
    if (nf_stop == nf_strt)
      {return (int_4)-1;}
  }

  // loop over all indices
  //
  for (int_4 nf=nf_strt; nf>=nf_stop; nf--) {

    int_4 num_matches = (int_4)0;
    int_4 n = nf;
    for (int_4 counter=0; counter<len_p; counter++) {

      // check the current frame
      //
      if (cb_d[n] == pattern_p[counter])
	{num_matches++;}

      // decrement the position
      //
      n = cb_increment_cc(n, (int_4)-1);
    }

    // check to see if we have found a pattern
    //
    if (num_matches == len_p)
      {return nf;}
  }

  // exit gracefully - no pattern found
  //
  return (int_4)-1;
}

