// file:/signal_detector/src_01/sigdp01_cb_4.cc
//

// isip include files
//
#include "signal_detector_01.h"
#include "signal_detector_01_constants.h"

// method: cb_smooth_signal_cc
//
// arguments: none
//
// return:
//  a logical_1 indicating the status
//
// this method removes short bursts of noise that do not meet the
// duration constraints.
//
// this algorithm works as follows:
//  (1) checks from the current position if there are min_utt_dur consecutive
//      frames assigned to state SIGN
//  (2) scans backward to find another min_utt_dur consecutive frames
//      assigned to state SIGN previous to the frames in step (1)
//  (3) if the time difference between these two groups of frames
//      is less than min_utt_sep, assign them to SIGN
//
logical_1 Signal_detector_01::cb_smooth_signal_cc() {

  // check from the current position
  //
  int_4 nf = cur_frm_d;
  for (int_4 counter=0; counter<i_min_utt_dur_d; counter++) {
    if (cb_d[nf] != SIGD_STATE_SIGN)
      {return ISIP_FALSE;}
    nf = cb_increment_cc(nf, (int_4)-1);
    if (nf == rlse_frm_d)
      {return ISIP_FALSE;}
  }

  // now, seek backwards until we find a state not equal to SIGN
  //
  while (nf != rlse_frm_d) {
    if (cb_d[nf] != SIGD_STATE_SIGN)
      {break;}
    nf = cb_increment_cc(nf, (int_4)-1);
  }
  if (nf == rlse_frm_d)
    {return ISIP_FALSE;}

  // scan backwards for another pattern of SIGD_STATE_SIGN
  //
  int_4 nf_stop = cb_scan_for_pattern_cc(sig_patn_d, sig_patn_len_d, nf);
  if (nf_stop < (int_4)0)
    {return ISIP_FALSE;}

  // compute the actual duration - note that this is a little tricky
  // because of the wraparound properties associated with a circular
  // buffer
  //
  int_4 delta = cb_diff_cc(nf, nf_stop);

  // if the duration of the region between these two patterns
  // is less than min_utt_sep, assign it to val_p
  //
  if (delta < i_min_utt_sep_d) {
    while (delta > (int_4)0) {
      cb_d[nf] = SIGD_STATE_SIGN;
      delta--;
      nf = cb_increment_cc(nf, (int_4)-1);
    }
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
