// file: signal_detector/src_01/sigdp01_cb_8.cc
//

// isip include files
//
#include "signal_detector_01.h"
#include "signal_detector_01_constants.h"

// method: cb_release_cc
//
// arguments:
//  int_4 frame_index_p: (input) frame index around which the circular
//				 buffer can be released
//
// return:
//  a logical_1 indicating the status
//
// this method starts at frame_index_p, which is assumed to be a
// noise state, and works backward until the first non-noise state is found.
// The release frame pointer, rlse_frm_d, is then set to this index.
//
// note that the portion of the buffer from the end of the last utterance
// to the current position, which is a run of NOIS states, is preserved.
// this may be useful in determining the endpoints of the next utterance.
//
logical_1 Signal_detector_01::cb_release_cc(int_4 frame_index_p) {

  // seek backwards to the first non-noise state
  //
  while (cb_d[frame_index_p] == SIGD_STATE_NOIS) {
    frame_index_p = cb_increment_cc(frame_index_p, (int_4)-1);
  }

  // increment by one
  //
  rlse_frm_d = cb_increment_cc(frame_index_p, (int_4)1);

  // set the state of the detector to noise
  //
  utt_in_progress_d = ISIP_FALSE;

  // exit gracefully
  //
  return ISIP_TRUE;
}
