// file: signal_detector/src_01/sigdp01_cb_9.cc
//

// isip include files
//
#include "signal_detector_01.h"
#include "signal_detector_01_constants.h"

// method: cb_remove_short_signal_bursts_cc
//
// arguments: none
//
// return:
//  a logical_1 indicating the status
//
// this method removes short bursts of signal that do not meet the
// duration constraints.
//
// this algorithm works as follows:
//  (1) checks from the current position if there are min_utt_sep_dur
//      consecutive frames assigned to state NOIS
//  (2) scans backward to find the next min_sep_dur consecutive frames
//      assigned to NOIS states
//  (3) checks if the time difference between these two groups of frames
//      is less than min_utt_sep, assigns them to NOIS
//
logical_1 Signal_detector_01::cb_remove_short_signal_bursts_cc() {

  // check from the current position for a run of noise states
  //
  int_4 nf = cur_frm_d;
  for (int_4 counter=0; counter<i_min_utt_sep_d; counter++) {
    if (cb_d[nf] != SIGD_STATE_NOIS)
      {return ISIP_FALSE;}
    nf = cb_increment_cc(nf, (int_4)-1);
    if (nf == rlse_frm_d)
      {return ISIP_FALSE;}
  }

  // check if the next state is non-NOIS
  //
  int_4 start_index = cb_increment_cc(nf, (int_4)-1);
  if ((start_index == rlse_frm_d) || (cb_d[start_index] == SIGD_STATE_NOIS))
    {return ISIP_FALSE;}

  // find the next run of NOIS states
  //
  int_4 end_index = start_index;
  int_4 last_index = cb_increment_cc(rlse_frm_d, i_min_utt_sep_d);

  while (end_index != last_index) {

    // count the number of frames that are assigned to the state NOIS
    //
    int_4 num_states_nois = (int_4)0;
    nf = end_index;
    int_4 counter = (int_4)0;

    while (counter<i_min_utt_sep_d) {
      if (cb_d[nf] != SIGD_STATE_NOIS)
	{break;}
      else
	{num_states_nois++;}

      // update counters
      //
      counter++;
      nf = cb_increment_cc(nf, (int_4)-1);
    }

    // check if we have a match
    //
    if (num_states_nois == i_min_utt_sep_d) {

      // check the duration of the burst
      //
      if (cb_diff_cc(start_index, end_index) < i_min_utt_dur_d) {

	// assign the states to NOIS
	//
	while (start_index != end_index) {
	  cb_d[start_index] = SIGD_STATE_NOIS;
	  start_index = cb_increment_cc(start_index, (int_4)-1);
	}
	
	// exit
	//
	return ISIP_TRUE;
      }
    }

    // decrement counter
    //
    end_index = cb_increment_cc(end_index, (int_4)-1);
  }

  // exit gracefully - no bursts found
  //
  return ISIP_FALSE;
}
