// file:signal_detector/src_01/sigdp01_egy_0.cc
//

// isip include files
//
#include "signal_detector_01.h"
#include "signal_detector_01_constants.h"

// method: compute_energy_cc
//
// arguments:
//  void* buf_p: (input) signal buffer
//
// return:
//  a float_4 containing the energy value
//
// note that this method debiases the data
//
float_4 Signal_detector_01::compute_energy_cc(void* buf_p) {

  // declare local variables
  //
  int_2* buf = (int_2*)buf_p;
  float_4* x_pre_off;
  float_4 xn;

  // find the offset in x_pre_d
  //
  if (i_window_dur_d > i_frame_dur_d)
    {x_pre_off = x_pre_d + (i_window_dur_d - i_frame_dur_d);}
  else
    {x_pre_off = x_pre_d;}

  // load the signal buffer
  //
  buf += chan_d;

  for (int_4 i=0; i<i_frame_dur_d; i++) {
    
    // grab a sample
    //
    xn = (float_4)(*buf) * SIGD_SAMPLE_SCALE_FACTOR;

    // preemphasize
    //
    *x_pre_off = xn - preemphasis_d * xn1_d;
    
    // shift the filter
    //
    xn1_d = xn;
    
    // increment counters
    //
    buf += num_chan_d;
    x_pre_off++;
  }

  // compute energy - do this backward so we don't have to worry
  //   when window duration is less than frame duration
  //
  float_4 sum = (float_4)0.0;
  float_4 avg = (float_4)0.0;
  x_pre_off = x_pre_d + i_window_dur_d - 1;
    
  for (int_4 i=0; i<i_window_dur_d; i++) {
    avg += (*x_pre_off); 
    sum += (*x_pre_off) * (*x_pre_off);
    x_pre_off--;
  }

  float_4 egy = win_dur_scale_factor_d *
    (sum - win_dur_scale_factor_d * avg * avg);

  // shift the signal
  //
  if (i_window_dur_d > i_frame_dur_d) {
    int_4 num_to_move = i_window_dur_d - i_frame_dur_d;
    x_pre_off = x_pre_d + i_frame_dur_d;
    memcpy(x_pre_d, x_pre_off, num_to_move*sizeof(float_4));
  }

  // exit gracefully - return the energy in dB
  //
  if (egy < ISIP_DB_POW_MIN_VALUE)
    {egy = ISIP_DB_POW_MIN_VALUE;}

  return ISIP_DB_POW_SCALE_FACTOR * log10(egy);
}

